'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var algorithm = require('@lumino/algorithm');
var collections = require('@lumino/collections');
var disposable = require('@lumino/disposable');
var messaging = require('@lumino/messaging');
var signaling = require('@lumino/signaling');
var schema = require('./schema.js');
var table = require('./table.js');
var utilities = require('./utilities.js');
require('tslib');

// Copyright (c) Jupyter Development Team.
/**
 * A multi-user collaborative datastore.
 *
 * #### Notes
 * A store is structured in a maximally flat way using a hierarchy
 * of tables, records, and fields. Internally, the object graph is
 * synchronized among all users via CRDT algorithms.
 *
 * https://en.wikipedia.org/wiki/Conflict-free_replicated_data_type
 * https://hal.inria.fr/file/index/docid/555588/filename/techreport.pdf
 */
exports.Datastore = /** @class */ (function () {
    /**
     * Create a new datastore.
     *
     * @param id - The unique id of the datastore.
     * @param tables - The tables of the datastore.
     */
    function Datastore(context, tables, adapter, transactionIdFactory) {
        this._cemetery = {};
        this._disposed = false;
        this._changed = new signaling.Signal(this);
        this._transactionQueue = new collections.LinkedList();
        this._context = context;
        this._tables = tables;
        this._adapter = adapter || null;
        this._transactionIdFactory = transactionIdFactory || utilities.createDuplexId;
        if (this._adapter) {
            this._adapter.onRemoteTransaction = this._onRemoteTransaction.bind(this);
            this._adapter.onUndo = this._onUndo.bind(this);
            this._adapter.onRedo = this._onRedo.bind(this);
        }
    }
    /**
     * Create a new datastore.
     *
     * @param options - The options for creating the datastore
     *
     * @returns A new datastore table.
     *
     * @throws An exception if any of the schema definitions are invalid.
     */
    Datastore.create = function (options) {
        var schemas = options.schemas;
        // Throws an error for invalid schemas:
        Private.validateSchemas(schemas);
        var context = {
            inTransaction: false,
            transactionId: '',
            version: 0,
            storeId: options.id,
            change: {},
            patch: {}
        };
        var tables = new collections.BPlusTree(Private.recordCmp);
        if (options.restoreState) {
            // If passed state to restore, pass the intital state to recreate each
            // table
            var state_1 = JSON.parse(options.restoreState);
            tables.assign(algorithm.map(schemas, function (s) {
                return table.Table.recreate(s, context, state_1[s.id] || []);
            }));
        }
        else {
            // Otherwise, simply create a new, empty table
            tables.assign(algorithm.map(schemas, function (s) {
                return table.Table.create(s, context);
            }));
        }
        return new Datastore(context, tables, options.adapter);
    };
    /**
     * Dispose of the resources held by the datastore.
     */
    Datastore.prototype.dispose = function () {
        // Bail if already disposed.
        if (this._disposed) {
            return;
        }
        this._disposed = true;
        signaling.Signal.clearData(this);
        this._adapter = null;
    };
    Object.defineProperty(Datastore.prototype, "isDisposed", {
        /**
         * Whether the datastore has been disposed.
         */
        get: function () {
            return this._disposed;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Datastore.prototype, "changed", {
        /**
         * A signal emitted when changes are made to the store.
         *
         * #### Notes
         * This signal is emitted either at the end of a local mutation,
         * or after a remote mutation has been applied. The storeId can
         * be used to determine its source.
         *
         * The payload represents the set of local changes that were made
         * to bring the store to its current state.
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._changed;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Datastore.prototype, "id", {
        /**
         * The unique id of the store.
         *
         * #### Notes
         * The id is unique among all other collaborating peers.
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._context.storeId;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Datastore.prototype, "inTransaction", {
        /**
         * Whether a transaction is currently in progress.
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._context.inTransaction;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Datastore.prototype, "version", {
        /**
         * The current version of the datastore.
         *
         * #### Notes
         * This version is automatically increased for each transaction
         * to the store. However, it might not increase linearly (i.e.
         * it might make jumps).
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._context.version;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create an iterator over all the tables of the datastore.
     *
     * @returns An iterator.
     */
    Datastore.prototype.iter = function () {
        return this._tables.iter();
    };
    /**
     * Get the table for a particular schema.
     *
     * @param schema - The schema of interest.
     *
     * @returns The table for the specified schema.
     *
     * @throws An exception if no table exists for the given schema.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    Datastore.prototype.get = function (schema) {
        var t = this._tables.get(schema.id, Private.recordIdCmp);
        if (t === undefined) {
            throw new Error("No table found for schema with id: ".concat(schema.id));
        }
        return t;
    };
    /**
     * Begin a new transaction in the store.
     *
     * @returns The id of the new transaction
     *
     * @throws An exception if a transaction is already in progress.
     *
     * #### Notes
     * This will allow the state of the store to be mutated
     * thorugh the `update` method on the individual tables.
     *
     * After the updates are completed, `endTransaction` should
     * be called.
     */
    Datastore.prototype.beginTransaction = function () {
        var newVersion = this._context.version + 1;
        var id = this._transactionIdFactory(newVersion, this.id);
        this._initTransaction(id, newVersion);
        messaging.MessageLoop.postMessage(this, new messaging.ConflatableMessage('transaction-begun'));
        return id;
    };
    /**
     * Completes a transaction.
     *
     * #### Notes
     * This completes a transaction previously started with
     * `beginTransaction`. If a change has occurred, the
     * `changed` signal will be emitted.
     */
    Datastore.prototype.endTransaction = function () {
        this._finalizeTransaction();
        var _a = this._context, patch = _a.patch, change = _a.change, storeId = _a.storeId, transactionId = _a.transactionId, version = _a.version;
        // Possibly broadcast the transaction to collaborators.
        if (this._adapter && !Private.isPatchEmpty(patch)) {
            this._adapter.broadcast({
                id: transactionId,
                storeId: storeId,
                patch: patch,
                version: version
            });
        }
        // Add the transation to the cemetery to indicate it is visible.
        this._cemetery[transactionId] = 1;
        // Emit a change signal
        if (!Private.isChangeEmpty(this._context.change)) {
            this._changed.emit({
                storeId: storeId,
                transactionId: transactionId,
                type: 'transaction',
                change: change
            });
        }
    };
    /**
     * Handle a message.
     */
    Datastore.prototype.processMessage = function (msg) {
        switch (msg.type) {
            case 'transaction-begun':
                if (this._context.inTransaction) {
                    console.warn("Automatically ending transaction (did you forget to end it?): ".concat(this._context.transactionId));
                    this.endTransaction();
                }
                break;
            case 'queued-transaction':
                this._processQueue();
                break;
        }
    };
    /**
     * Undo a patch that was previously applied.
     *
     * @param transactionId - The transaction to undo.
     *
     * @returns A promise which resolves when the action is complete.
     *
     * @throws An exception if `undo` is called during a mutation, or if no
     *   server adapter has been set for the datastore.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    Datastore.prototype.undo = function (transactionId) {
        if (!this._adapter) {
            throw Error('No server adapter has been set for the datastore');
        }
        if (this.inTransaction) {
            throw Error('Cannot undo during a transaction');
        }
        return this._adapter.undo(transactionId);
    };
    /**
     * Redo a patch that was previously undone.
     *
     * @param transactionId - The transaction to redo.
     *
     * @returns A promise which resolves when the action is complete.
     *
     * @throws An exception if `redo` is called during a mutation, or if no
     *   server adapter has been set for the datastore.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    Datastore.prototype.redo = function (transactionId) {
        if (!this._adapter) {
            throw Error('No server adapter has been set for the datastore');
        }
        if (this.inTransaction) {
            throw Error('Cannot redo during a transaction');
        }
        return this._adapter.redo(transactionId);
    };
    Object.defineProperty(Datastore.prototype, "adapter", {
        /**
         * The handler for broadcasting transactions to peers.
         */
        get: function () {
            return this._adapter;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Serialize the state of the datastore to a string.
     *
     * @returns The serialized state.
     */
    Datastore.prototype.toString = function () {
        return JSON.stringify(algorithm.toObject(algorithm.map(this, function (table) {
            return [table.schema.id, algorithm.toArray(table)];
        })));
    };
    /**
     * Handle a transaction from the server adapter.
     */
    Datastore.prototype._onRemoteTransaction = function (transaction) {
        this._processTransaction(transaction, 'transaction');
    };
    /**
     * Handle an undo from the server adapter.
     */
    Datastore.prototype._onUndo = function (transaction) {
        this._processTransaction(transaction, 'undo');
    };
    /**
     * Handle a redo from the server adapter.
     */
    Datastore.prototype._onRedo = function (transaction) {
        this._processTransaction(transaction, 'redo');
    };
    /**
     * Apply a transaction to the datastore.
     *
     * @param transactionApplication - The data of the transaction.
     *
     * @throws An exception if `processTransaction` is called during a mutation.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted.
     */
    Datastore.prototype._processTransaction = function (transaction, type) {
        var _this = this;
        var storeId = transaction.storeId, patch = transaction.patch;
        try {
            this._initTransaction(transaction.id, Math.max(this._context.version, transaction.version));
        }
        catch (e) {
            // Already in a transaction. Put the transaction in the queue to apply
            // later.
            this._queueTransaction(transaction, type);
            return;
        }
        var change = {};
        try {
            algorithm.each(algorithm.iterItems(patch), function (_a) {
                var schemaId = _a[0], tablePatch = _a[1];
                var table$1 = _this._tables.get(schemaId, Private.recordIdCmp);
                if (table$1 === undefined) {
                    console.warn("Missing table for schema id '".concat(schemaId, "' in transaction '").concat(transaction.id, "'"));
                    _this._finalizeTransaction();
                    return;
                }
                if (type === 'transaction' || type === 'redo') {
                    var count = _this._cemetery[transaction.id];
                    if (count === undefined) {
                        _this._cemetery[transaction.id] = 1;
                        change[schemaId] = table.Table.patch(table$1, tablePatch);
                        return;
                    }
                    _this._cemetery[transaction.id] = count + 1;
                    // If the transaction is just now positive, apply it to the store.
                    if (_this._cemetery[transaction.id] === 1) {
                        change[schemaId] = table.Table.patch(table$1, tablePatch);
                        return;
                    }
                }
                else {
                    var count = _this._cemetery[transaction.id];
                    if (count === undefined) {
                        _this._cemetery[transaction.id] = -1;
                        return;
                    }
                    _this._cemetery[transaction.id] = count - 1;
                    // If the transaction hasn't already been unapplied, do so.
                    if (_this._cemetery[transaction.id] === 0) {
                        change[schemaId] = table.Table.unpatch(table$1, tablePatch);
                    }
                }
            });
        }
        finally {
            this._finalizeTransaction();
        }
        if (!Private.isChangeEmpty(change)) {
            this._changed.emit({
                storeId: storeId,
                transactionId: transaction.id,
                type: type,
                change: change
            });
        }
    };
    /**
     * Queue a transaction for later application.
     *
     * @param transaction - the transaction to queue.
     */
    Datastore.prototype._queueTransaction = function (transaction, type) {
        this._transactionQueue.addLast([transaction, type]);
        messaging.MessageLoop.postMessage(this, new messaging.ConflatableMessage('queued-transaction'));
    };
    /**
     * Process all transactions currently queued.
     */
    Datastore.prototype._processQueue = function () {
        var queue = this._transactionQueue;
        // If the transaction queue is empty, bail.
        if (queue.isEmpty) {
            return;
        }
        // Add a sentinel value to the end of the queue. The queue will
        // only be processed up to the sentinel. Transactions added during
        // this cycle will execute on the next cycle.
        var sentinel = {};
        queue.addLast(sentinel);
        // Enter the processing loop.
        // eslint-disable-next-line no-constant-condition
        while (true) {
            // Remove the first transaction in the queue.
            var _a = queue.removeFirst(), transaction = _a[0], type = _a[1];
            // If the value is the sentinel, exit the loop.
            if (transaction === sentinel) {
                return;
            }
            // Apply the transaction.
            this._processTransaction(transaction, type);
        }
    };
    /**
     * Reset the context state for a new transaction.
     *
     * @param id - The id of the new transaction.
     * @param newVersion - The version of the datastore after the transaction.
     *
     * @throws An exception if a transaction is already in progress.
     */
    Datastore.prototype._initTransaction = function (id, newVersion) {
        var context = this._context;
        if (context.inTransaction) {
            throw new Error("Already in a transaction: ".concat(this._context.transactionId));
        }
        context.inTransaction = true;
        context.change = {};
        context.patch = {};
        context.transactionId = id;
        context.version = newVersion;
    };
    /**
     * Finalize the context state for a transaction in progress.
     *
     * @throws An exception if no transaction is in progress.
     */
    Datastore.prototype._finalizeTransaction = function () {
        var context = this._context;
        if (!context.inTransaction) {
            throw new Error('No transaction in progress.');
        }
        context.inTransaction = false;
    };
    return Datastore;
}());
/**
 * The namespace for the `Datastore` class statics.
 */
(function (Datastore) {
    /**
     * A helper function to wrap an update to the datastore in calls to
     * `beginTransaction` and `endTransaction`.
     *
     * @param datastore: the datastore to which to apply the update.
     *
     * @param update: A function that performs the update on the datastore.
     *   The function is called with a transaction id string, in case the
     *   user wishes to store the transaction ID for later use.
     *
     * @returns the transaction ID.
     *
     * #### Notes
     * If the datastore is already in a transaction, this does not attempt
     * to start a new one, and returns an empty string for the transaction
     * id. This allows for transactions to be composed a bit more easily.
     */
    function withTransaction(datastore, update) {
        var id = '';
        if (!datastore.inTransaction) {
            id = datastore.beginTransaction();
        }
        try {
            update(id);
        }
        finally {
            if (id) {
                datastore.endTransaction();
            }
        }
        return id;
    }
    Datastore.withTransaction = withTransaction;
    /**
     * Get a given table by its location.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: The table location.
     *
     * @returns the table.
     */
    function getTable(datastore, loc) {
        return datastore.get(loc.schema);
    }
    Datastore.getTable = getTable;
    /**
     * Get a given record by its location.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: The record location.
     *
     * @returns the record, or undefined if it does not exist.
     */
    function getRecord(datastore, loc) {
        return datastore.get(loc.schema).get(loc.record);
    }
    Datastore.getRecord = getRecord;
    /**
     * Get a given field by its location.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @returns the field in question.
     *
     * #### Notes
     * This will throw an error if the record does not exist in the given table.
     */
    function getField(datastore, loc) {
        var record = datastore.get(loc.schema).get(loc.record);
        if (!record) {
            throw Error("The record ".concat(loc.record, " could not be found"));
        }
        return record[loc.field];
    }
    Datastore.getField = getField;
    /**
     * Update a table.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: the table location.
     *
     * @param update: the update to the table.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateTable(datastore, loc, update) {
        var table = datastore.get(loc.schema);
        table.update(update);
    }
    Datastore.updateTable = updateTable;
    /**
     * Update a record in a table.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: the record location.
     *
     * @param update: the update to the record.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateRecord(datastore, loc, update) {
        var _a;
        var table = datastore.get(loc.schema);
        table.update((_a = {},
            _a[loc.record] = update,
            _a));
    }
    Datastore.updateRecord = updateRecord;
    /**
     * Update a field in a table.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @param update: the update to the field.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateField(datastore, loc, update) {
        var _a, _b;
        var table = datastore.get(loc.schema);
        // TODO: this cast may be made unnecessary once microsoft/TypeScript#13573
        // is fixed, possibly by microsoft/TypeScript#26797 lands.
        table.update((_a = {},
            _a[loc.record] = (_b = {},
                _b[loc.field] = update,
                _b),
            _a));
    }
    Datastore.updateField = updateField;
    /**
     * Listen to changes in a table. Changes to other tables are ignored.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: the table location.
     *
     * @param slot: a callback function to invoke when the table changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenTable(datastore, loc, slot, thisArg) {
        // A wrapper change signal connection function.
        var wrapper = function (source, args) {
            // Ignore changes that don't match the requested record.
            if (!args.change[loc.schema.id]) {
                return;
            }
            // Otherwise, call the slot.
            var tc = args.change[loc.schema.id];
            slot.bind(thisArg)(source, tc);
        };
        datastore.changed.connect(wrapper);
        return new disposable.DisposableDelegate(function () {
            datastore.changed.disconnect(wrapper);
        });
    }
    Datastore.listenTable = listenTable;
    /**
     * Listen to changes in a record in a table. Changes to other tables and
     * other records in the same table are ignored.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: the record location.
     *
     * @param slot: a callback function to invoke when the record changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenRecord(datastore, loc, slot, thisArg) {
        // A wrapper change signal connection function.
        var wrapper = function (source, args) {
            // Ignore changes that don't match the requested record.
            if (!args.change[loc.schema.id] ||
                !args.change[loc.schema.id][loc.record]) {
                return;
            }
            // Otherwise, call the slot.
            var tc = args.change[loc.schema.id];
            slot.bind(thisArg)(source, tc[loc.record]);
        };
        datastore.changed.connect(wrapper);
        return new disposable.DisposableDelegate(function () {
            datastore.changed.disconnect(wrapper);
        });
    }
    Datastore.listenRecord = listenRecord;
    /**
     * Listen to changes in a fields in a table. Changes to other tables, other
     * records in the same table, and other fields in the same record are ignored.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @param slot: a callback function to invoke when the field changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenField(datastore, loc, slot, thisArg) {
        var wrapper = function (source, args) {
            // Ignore changes that don't match the requested field.
            if (!args.change[loc.schema.id] ||
                !args.change[loc.schema.id][loc.record] ||
                !args.change[loc.schema.id][loc.record][loc.field]) {
                return;
            }
            // Otherwise, call the slot.
            var tc = args.change[loc.schema.id];
            slot.bind(thisArg)(source, tc[loc.record][loc.field]);
        };
        datastore.changed.connect(wrapper);
        return new disposable.DisposableDelegate(function () {
            datastore.changed.disconnect(wrapper);
        });
    }
    Datastore.listenField = listenField;
})(exports.Datastore || (exports.Datastore = {}));
var Private;
(function (Private) {
    /**
     * Validates all schemas, and throws an error if any are invalid.
     */
    function validateSchemas(schemas) {
        var errors = [];
        for (var _i = 0, schemas_1 = schemas; _i < schemas_1.length; _i++) {
            var s = schemas_1[_i];
            var err = schema.validateSchema(s);
            if (err.length) {
                errors.push("Schema '".concat(s.id, "' validation failed: \n").concat(err.join('\n')));
            }
        }
        if (errors.length) {
            throw new Error(errors.join('\n\n'));
        }
    }
    Private.validateSchemas = validateSchemas;
    /**
     * A three-way record comparison function.
     */
    function recordCmp(a, b) {
        return algorithm.StringExt.cmp(a.schema.id, b.schema.id);
    }
    Private.recordCmp = recordCmp;
    /**
     * A three-way record id comparison function.
     */
    function recordIdCmp(table, id) {
        return algorithm.StringExt.cmp(table.schema.id, id);
    }
    Private.recordIdCmp = recordIdCmp;
    /**
     * Checks if a patch is empty.
     */
    function isPatchEmpty(patch) {
        return Object.keys(patch).length === 0;
    }
    Private.isPatchEmpty = isPatchEmpty;
    /**
     * Checks if a change is empty.
     */
    function isChangeEmpty(change) {
        return Object.keys(change).length === 0;
    }
    Private.isChangeEmpty = isChangeEmpty;
})(Private || (Private = {}));
//# sourceMappingURL=datastore.js.map
