'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var algorithm = require('@lumino/algorithm');
var field = require('./field.js');
var utilities = require('./utilities.js');

/**
 * A field which represents a collaborative key:value map.
 */
var MapField = /** @class */ (function (_super) {
    tslib.__extends(MapField, _super);
    /**
     * Construct a new map field.
     *
     * @param options - The options for initializing the field.
     */
    function MapField(options) {
        if (options === void 0) { options = {}; }
        return _super.call(this, options) || this;
    }
    Object.defineProperty(MapField.prototype, "type", {
        /**
         * The discriminated type of the field.
         */
        get: function () {
            return 'map';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create the initial value for the field.
     *
     * @returns The initial value for the field.
     */
    MapField.prototype.createValue = function () {
        return {};
    };
    /**
     * Create the metadata for the field.
     *
     * @returns The metadata for the field.
     */
    MapField.prototype.createMetadata = function () {
        return { ids: {}, values: {} };
    };
    /**
     * Apply a user update to the field.
     *
     * @param args - The arguments for the update.
     *
     * @returns The result of applying the update.
     */
    MapField.prototype.applyUpdate = function (args) {
        // Unpack the arguments.
        var previous = args.previous, update = args.update, metadata = args.metadata, version = args.version, storeId = args.storeId;
        // Create the id for the values.
        var id = utilities.createDuplexId(version, storeId);
        // Create a clone of the previous value.
        var clone = tslib.__assign({}, previous);
        // Set up the previous and current change parts.
        var prev = {};
        var curr = {};
        // Iterate over the update.
        for (var key in update) {
            // Insert the update value into the metadata.
            var value = Private.insertIntoMetadata(metadata, key, id, update[key]);
            // Update the clone with the new value.
            if (value === null) {
                delete clone[key];
            }
            else {
                clone[key] = value;
            }
            // Update the previous change part.
            prev[key] = key in previous ? previous[key] : null;
            // Update the current change part.
            curr[key] = value;
        }
        // Create the change object.
        var change = { previous: prev, current: curr };
        // Create the patch object.
        var patch = { id: id, values: update };
        // Return the update result.
        return { value: clone, change: change, patch: patch };
    };
    /**
     * Apply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of applying the patch.
     */
    MapField.prototype.applyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Unpack the patch.
        var id = patch.id, values = patch.values;
        // Create a clone of the previous value.
        var clone = tslib.__assign({}, previous);
        // Set up the previous and current change parts.
        var prev = {};
        var curr = {};
        // Iterate over the values.
        for (var key in values) {
            // Insert the patch value into the metadata.
            var value = Private.insertIntoMetadata(metadata, key, id, values[key]);
            // Update the clone with the new value.
            if (value === null) {
                delete clone[key];
            }
            else {
                clone[key] = value;
            }
            // Update the previous change part.
            prev[key] = key in previous ? previous[key] : null;
            // Update the current change part.
            curr[key] = value;
        }
        // Create the change object.
        var change = { previous: prev, current: curr };
        // Return the patch result.
        return { value: clone, change: change };
    };
    /**
     * Apply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of applying the patch.
     */
    MapField.prototype.unapplyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Unpack the patch.
        var id = patch.id, values = patch.values;
        // Create a clone of the previous value.
        var clone = tslib.__assign({}, previous);
        // Set up the previous and current change parts.
        var prev = {};
        var curr = {};
        // Iterate over the values.
        for (var key in values) {
            // Remove the patch value from the metadata.
            var value = Private.removeFromMetadata(metadata, key, id);
            // Update the clone with the new value.
            if (value === null) {
                delete clone[key];
            }
            else {
                clone[key] = value;
            }
            // Update the previous change part.
            prev[key] = key in previous ? previous[key] : null;
            // Update the current change part.
            curr[key] = value;
        }
        // Create the change object.
        var change = { previous: prev, current: curr };
        // Return the patch result.
        return { value: clone, change: change };
    };
    /**
     * Merge two change objects into a single change object.
     *
     * @param first - The first change object of interest.
     *
     * @param second - The second change object of interest.
     *
     * @returns A new change object which represents both changes.
     */
    MapField.prototype.mergeChange = function (first, second) {
        var previous = tslib.__assign(tslib.__assign({}, second.previous), first.previous);
        var current = tslib.__assign(tslib.__assign({}, first.current), second.current);
        return { previous: previous, current: current };
    };
    /**
     * Merge two patch objects into a single patch object.
     *
     * @param first - The first patch object of interest.
     *
     * @param second - The second patch object of interest.
     *
     * @returns A new patch object which represents both patches.
     */
    MapField.prototype.mergePatch = function (first, second) {
        return { id: second.id, values: tslib.__assign(tslib.__assign({}, first.values), second.values) };
    };
    return MapField;
}(field.Field));
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * Insert a value into the map field metadata.
     *
     * @param metadata - The metadata of interest.
     *
     * @param key - The key of interest.
     *
     * @param id - The unique id for the value.
     *
     * @param value - The value to insert.
     *
     * @returns The current value for the key.
     *
     * #### Notes
     * If the id already exists, the old value will be overwritten.
     */
    function insertIntoMetadata(metadata, key, id, value) {
        // Fetch the id and value arrays for the given key.
        var ids = metadata.ids[key] || (metadata.ids[key] = []);
        var values = metadata.values[key] || (metadata.values[key] = []);
        // Find the insert index for the id.
        var i = algorithm.ArrayExt.lowerBound(ids, id, algorithm.StringExt.cmp);
        // Overwrite or insert the value as appropriate.
        if (i < ids.length && ids[i] === id) {
            values[i] = value;
        }
        else {
            algorithm.ArrayExt.insert(ids, i, id);
            algorithm.ArrayExt.insert(values, i, value);
        }
        // Return the current value for the key.
        return values[values.length - 1];
    }
    Private.insertIntoMetadata = insertIntoMetadata;
    /**
     * Remove a value from the map field metadata.
     *
     * @param metadata - The metadata of interest.
     *
     * @param key - The key of interest.
     *
     * @param id - The unique id for the value.
     *
     * @returns The current value for the key, or null if there is no value.
     *
     * #### Notes
     * If the id is not in the metadata, this is a no-op.
     */
    function removeFromMetadata(metadata, key, id) {
        // Fetch the id and value arrays for the given key.
        var ids = metadata.ids[key] || (metadata.ids[key] = []);
        var values = metadata.values[key] || (metadata.values[key] = []);
        // Find the insert index for the id.
        var i = algorithm.ArrayExt.lowerBound(ids, id, algorithm.StringExt.cmp);
        // Find and remove the index for the id.
        if (ids[i] === id) {
            algorithm.ArrayExt.removeAt(ids, i);
            algorithm.ArrayExt.removeAt(values, i);
        }
        // Return the current value for the key.
        return values.length ? values[values.length - 1] : null;
    }
    Private.removeFromMetadata = removeFromMetadata;
})(Private || (Private = {}));

exports.MapField = MapField;
//# sourceMappingURL=mapfield.js.map
