'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var algorithm = require('@lumino/algorithm');
var field = require('./field.js');
var utilities = require('./utilities.js');

/**
 * A field which represents a collaborative atomic value.
 */
var RegisterField = /** @class */ (function (_super) {
    tslib.__extends(RegisterField, _super);
    /**
     * Construct a new register field.
     *
     * @param options - The options for initializing the field.
     */
    function RegisterField(options) {
        var _this = _super.call(this, options) || this;
        _this.value = options.value;
        return _this;
    }
    Object.defineProperty(RegisterField.prototype, "type", {
        /**
         * The discriminated type of the field.
         */
        get: function () {
            return 'register';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create the initial value for the field.
     *
     * @returns The initial value for the field.
     */
    RegisterField.prototype.createValue = function () {
        return this.value;
    };
    /**
     * Create the metadata for the field.
     *
     * @returns The metadata for the field.
     */
    RegisterField.prototype.createMetadata = function () {
        return { ids: [], values: [] };
    };
    /**
     * Apply a user update to the field.
     *
     * @param args - The arguments for the update.
     *
     * @returns The result of applying the update.
     */
    RegisterField.prototype.applyUpdate = function (args) {
        // Unpack the arguments.
        var previous = args.previous, update = args.update, metadata = args.metadata, version = args.version, storeId = args.storeId;
        // Create the id for the value.
        var id = utilities.createDuplexId(version, storeId);
        // Insert the update value into the metadata.
        var value = Private.insertIntoMetadata(metadata, id, update);
        // Create the change object.
        var change = { previous: previous, current: value };
        // Create the patch object.
        var patch = { id: id, value: update };
        // Return the result of the update.
        return { value: value, change: change, patch: patch };
    };
    /**
     * Apply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of applying the patch.
     */
    RegisterField.prototype.applyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Insert the patch value into the metadata.
        var value = Private.insertIntoMetadata(metadata, patch.id, patch.value);
        // Create the change object.
        var change = { previous: previous, current: value };
        // Return the result of the patch.
        return { value: value, change: change };
    };
    /**
     * Unapply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of unapplying the patch.
     */
    RegisterField.prototype.unapplyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Remove the patch value from the metadata.
        var value = Private.removeFromMetadata(metadata, patch.id, this.value);
        // Create the change object.
        var change = { previous: previous, current: value };
        // Return the result of the patch.
        return { value: value, change: change };
    };
    /**
     * Merge two change objects into a single change object.
     *
     * @param first - The first change object of interest.
     *
     * @param second - The second change object of interest.
     *
     * @returns A new change object which represents both changes.
     */
    RegisterField.prototype.mergeChange = function (first, second) {
        return { previous: first.previous, current: second.current };
    };
    /**
     * Merge two patch objects into a single patch object.
     *
     * @param first - The first patch object of interest.
     *
     * @param second - The second patch object of interest.
     *
     * @returns A new patch object which represents both patches.
     */
    RegisterField.prototype.mergePatch = function (first, second) {
        return second;
    };
    return RegisterField;
}(field.Field));
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * Insert a value into the register field metadata.
     *
     * @param metadata - The metadata of interest.
     *
     * @param id - The unique id for the value.
     *
     * @param value - The value to insert.
     *
     * @returns The current value for the register field.
     *
     * #### Notes
     * If the id already exists, the old value will be overwritten.
     */
    function insertIntoMetadata(metadata, id, value) {
        // Unpack the metadata.
        var ids = metadata.ids, values = metadata.values;
        // Find the insert index for the id.
        var i = algorithm.ArrayExt.lowerBound(ids, id, algorithm.StringExt.cmp);
        // Overwrite or insert the value as appropriate.
        if (i < ids.length && ids[i] === id) {
            values[i] = value;
        }
        else {
            algorithm.ArrayExt.insert(ids, i, id);
            algorithm.ArrayExt.insert(values, i, value);
        }
        // Return the current value for the register field.
        return values[values.length - 1];
    }
    Private.insertIntoMetadata = insertIntoMetadata;
    /**
     * Remove a value from the register field metadata.
     *
     * @param metadata - The metadata of interest.
     *
     * @param id - The unique id for the value.
     *
     * @param initial - The default value for the field
     *
     * @returns The current value for the register field.
     *
     * #### Notes
     * If the id does not exist in the metadata, this is a no-op.
     */
    function removeFromMetadata(metadata, id, initial) {
        // Unpack the metadata.
        var ids = metadata.ids, values = metadata.values;
        // Find the remove index for the id.
        var i = algorithm.ArrayExt.lowerBound(ids, id, algorithm.StringExt.cmp);
        if (ids[i] === id) {
            algorithm.ArrayExt.removeAt(ids, i);
            algorithm.ArrayExt.removeAt(values, i);
        }
        // Return the current value for the register field.
        // If there are no values in the metadata, return the default value.
        return values.length ? values[values.length - 1] : initial;
    }
    Private.removeFromMetadata = removeFromMetadata;
})(Private || (Private = {}));

exports.RegisterField = RegisterField;
//# sourceMappingURL=registerfield.js.map
