'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var algorithm = require('@lumino/algorithm');
var collections = require('@lumino/collections');

/**
 * A datastore object which holds a collection of records.
 */
var Table = /** @class */ (function () {
    /**
     * Construct a new datastore table.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     */
    function Table(schema, context, records) {
        this._records = new collections.BPlusTree(Private.recordCmp);
        this.schema = schema;
        this._context = context;
        if (records) {
            this._records.assign(records);
        }
    }
    /**
     * @internal
     *
     * Create a new datastore table.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     *
     * @returns A new datastore table.
     */
    Table.create = function (schema, context) {
        return new Table(schema, context);
    };
    /**
     * @internal
     *
     * Create a new datastore table with a previously exported state.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     *
     * @returns A new datastore table.
     */
    Table.recreate = function (schema, context, records) {
        return new Table(schema, context, records);
    };
    /**
     * @internal
     *
     * Apply a patch to a datastore table.
     *
     * @param table - The table of interest.
     *
     * @param data - The patch to apply to the table.
     *
     * @returns The user-facing change to the table.
     */
    Table.patch = function (table, data) {
        // Create the change object.
        var tc = {};
        // Fetch common variables.
        var schema = table.schema;
        var records = table._records;
        var cmp = Private.recordIdCmp;
        // Iterate over the dataset.
        for (var id in data) {
            // Get or create the old record.
            var old = records.get(id, cmp) || Private.createRecord(schema, id);
            // Apply the patch and create the new record.
            var _a = Private.applyPatch(schema, old, data[id]), record = _a.record, change = _a.change;
            // Replace the old record in the table.
            records.insert(record);
            // Update the change object.
            tc[id] = change;
        }
        // Return the change object.
        return tc;
    };
    /**
     * @internal
     *
     * Unapply a patch to a datastore table, thereby undoing that patch.
     *
     * @param table - The table of interest.
     *
     * @param data - The patch to apply to the table.
     *
     * @returns The user-facing change to the table.
     */
    Table.unpatch = function (table, data) {
        // Create the change object.
        var tc = {};
        // Fetch common variables.
        var schema = table.schema;
        var records = table._records;
        var cmp = Private.recordIdCmp;
        // Iterate over the dataset.
        for (var id in data) {
            // Get or create the old record.
            var old = records.get(id, cmp) || Private.createRecord(schema, id);
            // Apply the patch and create the new record.
            var _a = Private.unapplyPatch(schema, old, data[id]), record = _a.record, change = _a.change;
            // Replace the old record in the table.
            records.insert(record);
            // Update the change object.
            tc[id] = change;
        }
        // Return the change object.
        return tc;
    };
    Object.defineProperty(Table.prototype, "isEmpty", {
        /**
         * Whether the table is empty.
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._records.isEmpty;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Table.prototype, "size", {
        /**
         * The size of the table.
         *
         * #### Complexity
         * `O(1)`
         */
        get: function () {
            return this._records.size;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create an iterator over the records in the table.
     *
     * @returns A new iterator over the table records.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    Table.prototype.iter = function () {
        return this._records.iter();
    };
    /**
     * Test whether the table has a particular record.
     *
     * @param id - The id of the record of interest.
     *
     * @returns `true` if the table has the record, `false` otherwise.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    Table.prototype.has = function (id) {
        return this._records.has(id, Private.recordIdCmp);
    };
    /**
     * Get the record for a particular id in the table.
     *
     * @param id - The id of the record of interest.
     *
     * @returns The record for the specified id, or `undefined` if no
     *   such record exists.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    Table.prototype.get = function (id) {
        return this._records.get(id, Private.recordIdCmp);
    };
    /**
     * Update one or more records in the table.
     *
     * @param data - The data for updating the records.
     *
     * #### Notes
     * If a specified record does not exist, it will be created.
     *
     * This method may only be called during a datastore transaction.
     */
    Table.prototype.update = function (data) {
        // Fetch the context.
        var context = this._context;
        // Ensure the update happens during a transaction.
        if (!context.inTransaction) {
            throw new Error('A table can only be updated during a transaction.');
        }
        // Fetch common variables.
        var schema = this.schema;
        var records = this._records;
        var cmp = Private.recordIdCmp;
        // Iterate over the data.
        for (var id in data) {
            // Get or create the old record.
            var old = records.get(id, cmp) || Private.createRecord(schema, id);
            // Apply the update and create the new record.
            var record = Private.applyUpdate(schema, old, data[id], context);
            // Replace the old record in the table.
            records.insert(record);
        }
    };
    return Table;
}());
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * A three-way record comparison function.
     */
    function recordCmp(a, b) {
        return algorithm.StringExt.cmp(a.$id, b.$id);
    }
    Private.recordCmp = recordCmp;
    /**
     * A three-way record id comparison function.
     */
    function recordIdCmp(record, id) {
        return algorithm.StringExt.cmp(record.$id, id);
    }
    Private.recordIdCmp = recordIdCmp;
    /**
     * Create a new record object.
     *
     * @param schema - The schema for the record.
     *
     * @param id - The unique id for the record.
     *
     * @returns A new default initialized record.
     */
    function createRecord(schema, id) {
        // Create the record and metadata objects.
        var record = {};
        var metadata = {};
        // Set the base record state.
        record.$id = id;
        record['@@metadata'] = metadata;
        // Populate the record and metadata.
        for (var name_1 in schema.fields) {
            var field = schema.fields[name_1];
            record[name_1] = field.createValue();
            metadata[name_1] = field.createMetadata();
        }
        // Return the new record.
        return record;
    }
    Private.createRecord = createRecord;
    /**
     * Apply an update to a record.
     *
     * @param schema - The schema for the record.
     *
     * @param record - The record of interest.
     *
     * @param update - The update to apply to the record.
     *
     * @param context - The datastore context.
     *
     * @returns A new record with the update applied.
     */
    function applyUpdate(schema, record, update, context) {
        // Fetch the version and store id.
        var version = context.version;
        var storeId = context.storeId;
        // Fetch or create the table change and patch.
        var tc = context.change[schema.id] || (context.change[schema.id] = {});
        var tp = context.patch[schema.id] || (context.patch[schema.id] = {});
        // Fetch or create the record change and patch.
        var rc = tc[record.$id] || (tc[record.$id] = {});
        var rp = tp[record.$id] || (tp[record.$id] = {});
        // Cast the record to a value object.
        var previous = record;
        // Fetch the record metadata.
        var metadata = record['@@metadata'];
        // Create a clone of the record.
        var clone = tslib.__assign({}, record);
        // Iterate over the update.
        for (var name_2 in update) {
            // Fetch the relevant field.
            var field = schema.fields[name_2];
            // Apply the update for the field.
            var _a = field.applyUpdate({
                previous: previous[name_2],
                update: update[name_2],
                metadata: metadata[name_2],
                version: version,
                storeId: storeId
            }), value = _a.value, change = _a.change, patch = _a.patch;
            // Assign the new value to the clone.
            clone[name_2] = value;
            // Merge the change if needed.
            if (name_2 in rc) {
                change = field.mergeChange(rc[name_2], change);
            }
            // Merge the patch if needed.
            if (name_2 in rp) {
                patch = field.mergePatch(rp[name_2], patch);
            }
            // Update the record change and patch for the field.
            rc[name_2] = change;
            rp[name_2] = patch;
        }
        // Return the new record.
        return clone;
    }
    Private.applyUpdate = applyUpdate;
    /**
     * Apply a patch to a record.
     *
     * @param schema - The schema for the record.
     *
     * @param record - The record of interest.
     *
     * @param patch - The patch to apply to the record.
     *
     * @return The result of applying the patch.
     */
    function applyPatch(schema, record, patch) {
        // Create the change object.
        var rc = {};
        // Cast the record to a value object.
        var previous = record;
        // Fetch the record metadata.
        var metadata = record['@@metadata'];
        // Create a clone of the record.
        var clone = tslib.__assign({}, record);
        // Iterate over the patch.
        for (var name_3 in patch) {
            // Fetch the relevant field.
            var field = schema.fields[name_3];
            // Apply the patch for the field.
            var _a = field.applyPatch({
                previous: previous[name_3],
                patch: patch[name_3],
                metadata: metadata[name_3]
            }), value = _a.value, change = _a.change;
            // Assign the new value to the clone.
            clone[name_3] = value;
            // Update the change object.
            rc[name_3] = change;
        }
        // Return the patch result.
        return { record: clone, change: rc };
    }
    Private.applyPatch = applyPatch;
    /**
     * Unapply a patch to a record.
     *
     * @param schema - The schema for the record.
     *
     * @param record - The record of interest.
     *
     * @param patch - The patch to unapply to the record.
     *
     * @return The result of unapplying the patch.
     */
    function unapplyPatch(schema, record, patch) {
        // Create the change object.
        var rc = {};
        // Cast the record to a value object.
        var previous = record;
        // Fetch the record metadata.
        var metadata = record['@@metadata'];
        // Create a clone of the record.
        var clone = tslib.__assign({}, record);
        // Iterate over the patch.
        for (var name_4 in patch) {
            // Fetch the relevant field.
            var field = schema.fields[name_4];
            // Apply the patch for the field.
            var _a = field.unapplyPatch({
                previous: previous[name_4],
                patch: patch[name_4],
                metadata: metadata[name_4]
            }), value = _a.value, change = _a.change;
            // Assign the new value to the clone.
            clone[name_4] = value;
            // Update the change object.
            rc[name_4] = change;
        }
        // Return the patch result.
        return { record: clone, change: rc };
    }
    Private.unapplyPatch = unapplyPatch;
})(Private || (Private = {}));

exports.Table = Table;
//# sourceMappingURL=table.js.map
