'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var algorithm = require('@lumino/algorithm');
var field = require('./field.js');
var utilities = require('./utilities.js');

/**
 * A field which represents collaborative text.
 */
var TextField = /** @class */ (function (_super) {
    tslib.__extends(TextField, _super);
    /**
     * Construct a new text field.
     *
     * @param options - The options for initializing the field.
     */
    function TextField(options) {
        if (options === void 0) { options = {}; }
        return _super.call(this, options) || this;
    }
    Object.defineProperty(TextField.prototype, "type", {
        /**
         * The discriminated type of the field.
         */
        get: function () {
            return 'text';
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create the initial value for the field.
     *
     * @returns The initial value for the field.
     */
    TextField.prototype.createValue = function () {
        return '';
    };
    /**
     * Create the metadata for the field.
     *
     * @returns The metadata for the field.
     */
    TextField.prototype.createMetadata = function () {
        return { ids: [], cemetery: {} };
    };
    /**
     * Apply a user update to the field.
     *
     * @param args - The arguments for the update.
     *
     * @returns The result of applying the update.
     */
    TextField.prototype.applyUpdate = function (args) {
        // Unpack the arguments.
        var previous = args.previous, update = args.update, metadata = args.metadata, version = args.version, storeId = args.storeId;
        // Set up a variable to hold the current value.
        var value = previous;
        // Set up the change and patch arrays.
        var change = [];
        var patch = [];
        // Coerce the update into an array of splices.
        if (Private.isSplice(update)) {
            update = [update];
        }
        // Iterate over the update.
        for (var _i = 0, update_1 = update; _i < update_1.length; _i++) {
            var splice = update_1[_i];
            // Apply the splice to the value.
            var obj = Private.applySplice(value, splice, metadata, version, storeId);
            // Update the change array.
            change.push(obj.change);
            // Update the patch array.
            patch.push(obj.patch);
            // Update the current value.
            value = obj.value;
        }
        // Return the update result.
        return { value: value, change: change, patch: patch };
    };
    /**
     * Apply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of applying the patch.
     */
    TextField.prototype.applyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Set up a variable to hold the current value.
        var value = previous;
        // Set up the change array.
        var change = [];
        // Iterate over the patch.
        for (var _i = 0, patch_1 = patch; _i < patch_1.length; _i++) {
            var part = patch_1[_i];
            // Apply the patch part to the value.
            var obj = Private.applyPatch(value, part, metadata);
            // Update the change array.
            change.push.apply(change, obj.change);
            // Update the current value.
            value = obj.value;
        }
        // Return the patch result.
        return { value: value, change: change };
    };
    /**
     * Unapply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of unapplying the patch.
     */
    TextField.prototype.unapplyPatch = function (args) {
        // Unpack the arguments.
        var previous = args.previous, patch = args.patch, metadata = args.metadata;
        // Set up a variable to hold the current value.
        var value = previous;
        // Set up the change array.
        var change = [];
        // Iterate over the patch.
        for (var _i = 0, patch_2 = patch; _i < patch_2.length; _i++) {
            var part = patch_2[_i];
            var reversed = {
                removedIds: part.insertedIds,
                insertedIds: part.removedIds,
                removedText: part.insertedText,
                insertedText: part.removedText
            };
            // Apply the patch part to the value.
            var obj = Private.applyPatch(value, reversed, metadata);
            // Update the change array.
            change.push.apply(change, obj.change);
            // Update the current value.
            value = obj.value;
        }
        // Return the patch result.
        return { value: value, change: change };
    };
    /**
     * Merge two change objects into a single change object.
     *
     * @param first - The first change object of interest.
     *
     * @param second - The second change object of interest.
     *
     * @returns A new change object which represents both changes.
     */
    TextField.prototype.mergeChange = function (first, second) {
        return tslib.__spreadArray(tslib.__spreadArray([], first, true), second, true);
    };
    /**
     * Merge two patch objects into a single patch object.
     *
     * @param first - The first patch object of interest.
     *
     * @param second - The second patch object of interest.
     *
     * @returns A new patch object which represents both patches.
     */
    TextField.prototype.mergePatch = function (first, second) {
        return tslib.__spreadArray(tslib.__spreadArray([], first, true), second, true);
    };
    return TextField;
}(field.Field));
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * A type-guard function for a text field update type.
     */
    function isSplice(value) {
        return !Array.isArray(value);
    }
    Private.isSplice = isSplice;
    /**
     * Apply a splice to a text field.
     *
     * @param value - The current value of the field.
     *
     * @param splice - The splice to apply to the field.
     *
     * @param metadata - The metadata for the field.
     *
     * @param version - The current datastore version.
     *
     * @param storeId - The unique id of the datastore.
     *
     * @returns The result of the splice operation.
     */
    function applySplice(value, splice, metadata, version, storeId) {
        // Unpack the splice.
        var index = splice.index, remove = splice.remove, text = splice.text;
        // Clamp the index to the string bounds.
        if (index < 0) {
            index = Math.max(0, index + value.length);
        }
        else {
            index = Math.min(index, value.length);
        }
        // Clamp the remove count to the string bounds.
        var count = Math.min(remove, value.length - index);
        // Fetch the lower and upper identifiers.
        var lower = index === 0 ? '' : metadata.ids[index - 1];
        var upper = index === value.length ? '' : metadata.ids[index];
        // Create the ids for the splice.
        var ids = utilities.createTriplexIds(text.length, version, storeId, lower, upper);
        // Apply the splice to the ids.
        var removedIds = spliceArray(metadata.ids, index, count, ids);
        // Compute the removed text.
        var removedText = value.slice(index, index + count);
        // Create the change object.
        var change = { index: index, removed: removedText, inserted: text };
        // Create the patch object.
        var patch = {
            removedIds: removedIds,
            removedText: removedText,
            insertedIds: ids,
            insertedText: text
        };
        // Compute the new value.
        value = value.slice(0, index) + text + value.slice(index + count);
        // Return the splice result.
        return { change: change, patch: patch, value: value };
    }
    Private.applySplice = applySplice;
    /**
     * Apply a patch to a text field.
     *
     * @param value - The current value of the field.
     *
     * @param patch - The patch part to apply to the field.
     *
     * @param metadata - The metadata for the field.
     *
     * @returns The user-facing change array for the patch.
     */
    function applyPatch(value, patch, metadata) {
        // Unpack the patch.
        var removedIds = patch.removedIds, insertedIds = patch.insertedIds, insertedText = patch.insertedText;
        // Set up the change array.
        var change = [];
        // Process the removed identifiers, if necessary.
        if (removedIds.length > 0) {
            // Chunkify the removed identifiers,
            // or increment the removed ids in the cemetery.
            var chunks = findRemovedChunks(removedIds, metadata);
            // Process the chunks.
            while (chunks.length > 0) {
                // Pop the last-most chunk.
                var _a = chunks.pop(), index = _a.index, count = _a.count;
                // Remove the identifiers from the metadata.
                metadata.ids.splice(index, count);
                // Compute the removed text
                var removed = value.slice(index, index + count);
                // Compute the new value.
                value = value.slice(0, index) + value.slice(index + count);
                // Add the change part to the change array.
                change.push({ index: index, removed: removed, inserted: '' });
            }
        }
        // Process the inserted identifiers, if necessary.
        if (insertedIds.length > 0) {
            // Chunkify the inserted identifiers, or decrement the removed
            // ids in the cemetery.
            var chunks = findInsertedChunks(insertedIds, insertedText, metadata);
            // Process the chunks.
            while (chunks.length > 0) {
                // Pop the last-most chunk.
                var _b = chunks.pop(), index = _b.index, ids = _b.ids, text = _b.text;
                // Insert the identifiers into the metadata.
                spliceArray(metadata.ids, index, 0, ids);
                // Insert the text into the value.
                value = value.slice(0, index) + text + value.slice(index);
                // Add the change part to the change array.
                change.push({ index: index, removed: '', inserted: text });
            }
        }
        // Return the change array.
        return { change: change, value: value };
    }
    Private.applyPatch = applyPatch;
    /**
     * Convert an array of identifiers into removal chunks.
     *
     * @param ids - The ids to remove from the metadta.
     *
     * @param metadata - The metadata for the text field.
     *
     * @returns The ordered chunks to remove.
     *
     * #### Notes
     * The metadata may be mutated if concurrently removed chunks are encountered.
     */
    function findRemovedChunks(ids, metadata) {
        // Set up the chunks array.
        var chunks = [];
        // Set up the iteration index.
        var i = 0;
        // Fetch the identifier array length.
        var n = ids.length;
        // Iterate over the identifiers to remove.
        while (i < n) {
            // Find the boundary identifier for the current id.
            var j = algorithm.ArrayExt.lowerBound(metadata.ids, ids[i], algorithm.StringExt.cmp);
            // If the boundary is at the end of the array, or if the boundary id
            // does not match the id we are looking for, then we are dealing with
            // a concurrently deleted value. In that case, increment its reference
            // in the cemetery and continue processing ids.
            if (j === metadata.ids.length || metadata.ids[j] !== ids[i]) {
                var count_1 = metadata.cemetery[ids[i]] || 0;
                metadata.cemetery[ids[i]] = count_1 + 1;
                i++;
                continue;
            }
            // Set up the chunk index.
            var index = j;
            // Set up the chunk count.
            var count = 0;
            // Find the extent of the chunk.
            while (i < n && algorithm.StringExt.cmp(ids[i], metadata.ids[j]) === 0) {
                count++;
                i++;
                j++;
            }
            // Add the chunk to the chunks array, or bump the id index.
            if (count > 0) {
                chunks.push({ index: index, count: count });
            }
            else {
                i++;
            }
        }
        // Return the computed chunks.
        return chunks;
    }
    /**
     * Convert arrays of identifiers and values into insert chunks.
     *
     * @param ids - The ids to be inserted.
     *
     * @param text - The text to be inserted.
     *
     * @param metadata - The metadata for the text field.
     *
     * @returns The ordered chunks to insert.
     *
     * #### Notes
     * The metadata may be mutated if concurrently removed chunks are encountered.
     */
    function findInsertedChunks(ids, text, metadata) {
        var indices = [];
        var insertIds = [];
        var insertText = '';
        for (var i = 0; i < ids.length; i++) {
            // Check if the id has been concurrently deleted. If so, update
            // the cemetery, and continue processing without inserting the id.
            if (checkCemeteryForInsert(ids[i], metadata.cemetery)) {
                continue;
            }
            // Add the id to the ids which will be actually inserted.
            insertIds.push(ids[i]);
            indices.push(algorithm.ArrayExt.lowerBound(metadata.ids, ids[i], algorithm.StringExt.cmp));
            insertText += text[i];
        }
        return chunkifyInsertions(insertIds, insertText, indices);
    }
    /**
     * Consolidate inserted IDs into a set of chunks so that we can splice them
     * into the existing value with a minimal number of splices.
     *
     * @param ids - The ids to be inserted.
     *
     * @param text - The text to be inserted. Should be the same length as ids.
     *
     * @param indices - The indices at which to insert the text. Should be the same length as ids.
     *
     * @returns The ordered chunks to insert.
     */
    function chunkifyInsertions(ids, text, indices) {
        // Set up the chunks array.
        var chunks = [];
        // Set up the loop over the ids to insert.
        var insertIndex;
        var i = 0;
        while (i < ids.length) {
            // Reset the insert chunk data
            var chunkIds = [];
            var chunkText = '';
            insertIndex = indices[i];
            // Find the extent of the chunk
            while (indices[i] === insertIndex && i < ids.length) {
                chunkIds.push(ids[i]);
                chunkText += text[i];
                i++;
            }
            if (chunkText) {
                chunks.push({ index: insertIndex, ids: chunkIds, text: chunkText });
            }
        }
        return chunks;
    }
    /**
     * Check if an id should be inserted, or if it has been concurrently deleted.
     *
     * @param id - the id to check.
     *
     * @param cemetery - the cemetery which determines whether the id should be inserted.
     *
     * @returns whether the id was found, indicating that it shouldn't be inserted.
     *
     * #### Notes
     * If the ID *is* found in the cemetery, its value in the cemetery is decremented,
     * reflecting that it is closer to being shown.
     */
    function checkCemeteryForInsert(id, cemetery) {
        var count = cemetery[id] || 0;
        if (count === 1) {
            delete cemetery[id];
            return true;
        }
        if (count > 1) {
            cemetery[id] = count - 1;
            return true;
        }
        return false;
    }
    /**
     * Splice data into an array.
     *
     * #### Notes
     * This is intentionally similar to Array.splice, but chunks the splices into
     * multiple splices so that it does not crash if the number of spliced IDs
     * is greater than the maximum number of arguments for a function.
     *
     * @param arr - the array on which to perform the splice.
     *
     * @param start - the start index for the splice.
     *
     * @param deleteCount - how many indices to remove.
     *
     * @param items - the items to splice into the array.
     *
     * @returns an array of the deleted elements.
     */
    function spliceArray(arr, start, deleteCount, items) {
        if (!items) {
            return arr.splice(start, deleteCount);
        }
        var size = 100000;
        if (items.length < size) {
            return arr.splice.apply(arr, tslib.__spreadArray([start, deleteCount || 0], items, false));
        }
        var deleted = arr.splice(start, deleteCount);
        var n = Math.floor(items.length / size);
        var idx = 0;
        for (var i = 0; i < n; i++, idx += size) {
            arr.splice.apply(arr, tslib.__spreadArray([idx, 0], items.slice(idx, size), false));
        }
        arr.splice.apply(arr, tslib.__spreadArray([idx, 0], items.slice(idx), false));
        return deleted;
    }
})(Private || (Private = {}));

exports.TextField = TextField;
//# sourceMappingURL=textfield.js.map
