'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
/*-----------------------------------------------------------------------------
| Copyright (c) 2014-2018, PhosphorJS Contributors
|
| Distributed under the terms of the BSD 3-Clause License.
|
| The full license is in the file LICENSE, distributed with this software.
|----------------------------------------------------------------------------*/
/**
 * Create a duplex string identifier.
 *
 * @param version - The datastore version for the duplex id.
 *
 * @param store - The datastore id for the duplex id.
 *
 * @returns A string duplex id for the given arguments.
 *
 * #### Notes
 * ID format: <6-byte version><4-byte storeId>
 */
function createDuplexId(version, store) {
    // Split the version into 16-bit values.
    var vc = version & 0xffff;
    var vb = (((version - vc) / 0x10000) | 0) & 0xffff;
    var va = (((version - vb - vc) / 0x100000000) | 0) & 0xffff;
    // Split the store id into 16-bit values.
    var sb = store & 0xffff;
    var sa = (((store - sb) / 0x10000) | 0) & 0xffff;
    // Convert the parts into a string identifier duplex.
    return String.fromCharCode(va, vb, vc, sa, sb);
}
/**
 * Create a triplex string identifier between two boundaries.
 *
 * @param version - The datastore version for the triplex id.
 *
 * @param store - The datastore id for the triplex id.
 *
 * @param lower - The lower triplex boundary identifier or `''`
 *   to represent the lowest-most boundary.
 *
 * @param upper - The upper triplex boundary identifier or `''`
 *   to represent the upper-most boundary.
 *
 * @returns A new triplex identifier between the two boundaries.
 *
 * #### Notes
 * ID format: <6-byte path><6-byte version><4-byte storeId> * (N >= 1)
 */
function createTriplexId(version, store, lower, upper) {
    // The maximum path in a triplex id.
    var MAX_PATH = 0xffffffffffff;
    // Set up the variable to hold the id.
    var id = '';
    // Fetch the triplet counts of the ids.
    var lowerCount = lower ? Private.idTripletCount(lower) : 0;
    var upperCount = upper ? Private.idTripletCount(upper) : 0;
    // Iterate over the id triplets.
    for (var i = 0, n = Math.max(lowerCount, upperCount); i < n; ++i) {
        // Fetch the lower identifier triplet, padding as needed.
        var lp = void 0;
        var lc = void 0;
        var ls = void 0;
        if (i >= lowerCount) {
            lp = 0;
            lc = 0;
            ls = 0;
        }
        else {
            lp = Private.idPathAt(lower, i);
            lc = Private.idVersionAt(lower, i);
            ls = Private.idStoreAt(lower, i);
        }
        // Fetch the upper identifier triplet, padding as needed.
        var up = void 0;
        var uc = void 0;
        var us = void 0;
        if (i >= upperCount) {
            up = upperCount === 0 ? MAX_PATH + 1 : 0;
            uc = 0;
            us = 0;
        }
        else {
            up = Private.idPathAt(upper, i);
            uc = Private.idVersionAt(upper, i);
            us = Private.idStoreAt(upper, i);
        }
        // If the triplets are the same, copy the triplet and continue.
        if (lp === up && lc === uc && ls === us) {
            id += Private.createTriplet(lp, lc, ls);
            continue;
        }
        // If the triplets are different, the well-ordered identifiers
        // assumption means that the lower triplet compares less than
        // the upper triplet. The task now is to find the nearest free
        // path slot among the remaining triplets.
        // If there is free space between the path portions of the
        // triplets, select a new path which falls between them.
        if (up - lp > 1) {
            var np_1 = Private.randomPath(lp + 1, up - 1);
            id += Private.createTriplet(np_1, version, store);
            return id.slice();
        }
        // Otherwise, copy the left triplet and reset the upper count
        // to zero so that the loop chooses the nearest available path
        // slot after the current lower triplet.
        id += Private.createTriplet(lp, lc, ls);
        upperCount = 0;
    }
    // If this point is reached, the lower and upper identifiers share
    // the same path but diverge based on the version or store id. It is
    // safe to insert anywhere in an extra triplet.
    var np = Private.randomPath(1, MAX_PATH);
    id += Private.createTriplet(np, version, store);
    return id.slice();
}
/**
 * Create the multiple triplex identifiers.
 *
 * @param n - The number of identifiers to create.
 *
 * @param version - The datastore version.
 *
 * @param store - The datastore id.
 *
 * @param lower - The lower boundary identifier, exclusive.
 *
 * @param uppper - The upper boundary identifier, exclusive.
 *
 * @returns The requested identifiers.
 */
function createTriplexIds(n, version, store, lower, upper) {
    // Initialize the identifiers array.
    var ids = [];
    // Loop the required number of times.
    while (ids.length < n) {
        // Create an identifier between the boundaries.
        var id = createTriplexId(version, store, lower, upper);
        // Add the identifier to the array.
        ids.push(id);
        // Update the lower boundary identifier.
        lower = id;
    }
    // Return the generated identifiers.
    return ids;
}
/**
 * The namespace for the module implementation details.
 */
var Private;
(function (Private) {
    /**
     * Create a string identifier triplet.
     *
     * @param path - The path value for the triplet.
     *
     * @param version - The version for the triplet.
     *
     * @param store - The store id for the triplet.
     *
     * @returns The string identifier triplet.
     */
    function createTriplet(path, version, store) {
        // Split the path into 16-bit values.
        var pc = path & 0xffff;
        var pb = (((path - pc) / 0x10000) | 0) & 0xffff;
        var pa = (((path - pb - pc) / 0x100000000) | 0) & 0xffff;
        // Split the version into 16-bit values.
        var vc = version & 0xffff;
        var vb = (((version - vc) / 0x10000) | 0) & 0xffff;
        var va = (((version - vb - vc) / 0x100000000) | 0) & 0xffff;
        // Split the store id into 16-bit values.
        var sb = store & 0xffff;
        var sa = (((store - sb) / 0x10000) | 0) & 0xffff;
        // Convert the parts into a string identifier triplet.
        return String.fromCharCode(pa, pb, pc, va, vb, vc, sa, sb);
    }
    Private.createTriplet = createTriplet;
    /**
     * Get the total number of path triplets in an identifier.
     *
     * @param id - The identifier of interest.
     *
     * @returns The total number of triplets in the id.
     */
    function idTripletCount(id) {
        return id.length >> 3;
    }
    Private.idTripletCount = idTripletCount;
    /**
     * Get the path value for a particular triplet.
     *
     * @param id - The string id of interest.
     *
     * @param i - The index of the triplet.
     *
     * @returns The path value for the specified triplet.
     */
    function idPathAt(id, i) {
        var j = i << 3;
        var a = id.charCodeAt(j + 0);
        var b = id.charCodeAt(j + 1);
        var c = id.charCodeAt(j + 2);
        return a * 0x100000000 + b * 0x10000 + c;
    }
    Private.idPathAt = idPathAt;
    /**
     * Get the version for a particular triplet.
     *
     * @param id - The identifier of interest.
     *
     * @param i - The index of the triplet.
     *
     * @returns The version for the specified triplet.
     */
    function idVersionAt(id, i) {
        var j = i << 3;
        var a = id.charCodeAt(j + 3);
        var b = id.charCodeAt(j + 4);
        var c = id.charCodeAt(j + 5);
        return a * 0x100000000 + b * 0x10000 + c;
    }
    Private.idVersionAt = idVersionAt;
    /**
     * Get the store id for a particular triplet.
     *
     * @param id - The identifier of interest.
     *
     * @param i - The index of the triplet.
     *
     * @returns The store id for the specified triplet.
     */
    function idStoreAt(id, i) {
        var j = i << 3;
        var a = id.charCodeAt(j + 6);
        var b = id.charCodeAt(j + 7);
        return a * 0x10000 + b;
    }
    Private.idStoreAt = idStoreAt;
    /**
     * Pick a path in the leading bucket of an inclusive range.
     *
     * @param min - The minimum allowed path, inclusive.
     *
     * @param max - The maximum allowed path, inclusive.
     *
     * @returns A random path in the leading bucket of the range.
     */
    function randomPath(min, max) {
        return min + Math.round(Math.random() * Math.sqrt(max - min));
    }
    Private.randomPath = randomPath;
})(Private || (Private = {}));

exports.createDuplexId = createDuplexId;
exports.createTriplexId = createTriplexId;
exports.createTriplexIds = createTriplexIds;
//# sourceMappingURL=utilities.js.map
