import { IIterable, IIterator } from '@lumino/algorithm';
import { IDisposable } from '@lumino/disposable';
import { IMessageHandler, Message } from '@lumino/messaging';
import { ISignal } from '@lumino/signaling';
import { Record } from './record';
import { Schema } from './schema';
import { IServerAdapter } from './serveradapter';
import { Table } from './table';
/**
 * A multi-user collaborative datastore.
 *
 * #### Notes
 * A store is structured in a maximally flat way using a hierarchy
 * of tables, records, and fields. Internally, the object graph is
 * synchronized among all users via CRDT algorithms.
 *
 * https://en.wikipedia.org/wiki/Conflict-free_replicated_data_type
 * https://hal.inria.fr/file/index/docid/555588/filename/techreport.pdf
 */
export declare class Datastore implements IDisposable, IIterable<Table<Schema>>, IMessageHandler {
    /**
     * Create a new datastore.
     *
     * @param options - The options for creating the datastore
     *
     * @returns A new datastore table.
     *
     * @throws An exception if any of the schema definitions are invalid.
     */
    static create(options: Datastore.IOptions): Datastore;
    /**
     * Dispose of the resources held by the datastore.
     */
    dispose(): void;
    /**
     * Whether the datastore has been disposed.
     */
    get isDisposed(): boolean;
    /**
     * A signal emitted when changes are made to the store.
     *
     * #### Notes
     * This signal is emitted either at the end of a local mutation,
     * or after a remote mutation has been applied. The storeId can
     * be used to determine its source.
     *
     * The payload represents the set of local changes that were made
     * to bring the store to its current state.
     *
     * #### Complexity
     * `O(1)`
     */
    get changed(): ISignal<Datastore, Datastore.IChangedArgs>;
    /**
     * The unique id of the store.
     *
     * #### Notes
     * The id is unique among all other collaborating peers.
     *
     * #### Complexity
     * `O(1)`
     */
    get id(): number;
    /**
     * Whether a transaction is currently in progress.
     *
     * #### Complexity
     * `O(1)`
     */
    get inTransaction(): boolean;
    /**
     * The current version of the datastore.
     *
     * #### Notes
     * This version is automatically increased for each transaction
     * to the store. However, it might not increase linearly (i.e.
     * it might make jumps).
     *
     * #### Complexity
     * `O(1)`
     */
    get version(): number;
    /**
     * Create an iterator over all the tables of the datastore.
     *
     * @returns An iterator.
     */
    iter(): IIterator<Table<Schema>>;
    /**
     * Get the table for a particular schema.
     *
     * @param schema - The schema of interest.
     *
     * @returns The table for the specified schema.
     *
     * @throws An exception if no table exists for the given schema.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    get<S extends Schema>(schema: S): Table<S>;
    /**
     * Begin a new transaction in the store.
     *
     * @returns The id of the new transaction
     *
     * @throws An exception if a transaction is already in progress.
     *
     * #### Notes
     * This will allow the state of the store to be mutated
     * thorugh the `update` method on the individual tables.
     *
     * After the updates are completed, `endTransaction` should
     * be called.
     */
    beginTransaction(): string;
    /**
     * Completes a transaction.
     *
     * #### Notes
     * This completes a transaction previously started with
     * `beginTransaction`. If a change has occurred, the
     * `changed` signal will be emitted.
     */
    endTransaction(): void;
    /**
     * Handle a message.
     */
    processMessage(msg: Message): void;
    /**
     * Undo a patch that was previously applied.
     *
     * @param transactionId - The transaction to undo.
     *
     * @returns A promise which resolves when the action is complete.
     *
     * @throws An exception if `undo` is called during a mutation, or if no
     *   server adapter has been set for the datastore.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    undo(transactionId: string): Promise<void>;
    /**
     * Redo a patch that was previously undone.
     *
     * @param transactionId - The transaction to redo.
     *
     * @returns A promise which resolves when the action is complete.
     *
     * @throws An exception if `redo` is called during a mutation, or if no
     *   server adapter has been set for the datastore.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    redo(transactionId: string): Promise<void>;
    /**
     * The handler for broadcasting transactions to peers.
     */
    get adapter(): IServerAdapter | null;
    /**
     * Serialize the state of the datastore to a string.
     *
     * @returns The serialized state.
     */
    toString(): string;
    /**
     * Create a new datastore.
     *
     * @param id - The unique id of the datastore.
     * @param tables - The tables of the datastore.
     */
    private constructor();
    /**
     * Handle a transaction from the server adapter.
     */
    private _onRemoteTransaction;
    /**
     * Handle an undo from the server adapter.
     */
    private _onUndo;
    /**
     * Handle a redo from the server adapter.
     */
    private _onRedo;
    /**
     * Apply a transaction to the datastore.
     *
     * @param transactionApplication - The data of the transaction.
     *
     * @throws An exception if `processTransaction` is called during a mutation.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted.
     */
    private _processTransaction;
    /**
     * Queue a transaction for later application.
     *
     * @param transaction - the transaction to queue.
     */
    private _queueTransaction;
    /**
     * Process all transactions currently queued.
     */
    private _processQueue;
    /**
     * Reset the context state for a new transaction.
     *
     * @param id - The id of the new transaction.
     * @param newVersion - The version of the datastore after the transaction.
     *
     * @throws An exception if a transaction is already in progress.
     */
    private _initTransaction;
    /**
     * Finalize the context state for a transaction in progress.
     *
     * @throws An exception if no transaction is in progress.
     */
    private _finalizeTransaction;
    private _adapter;
    private _cemetery;
    private _disposed;
    private _tables;
    private _context;
    private _changed;
    private _transactionIdFactory;
    private _transactionQueue;
}
/**
 * The namespace for the `Datastore` class statics.
 */
export declare namespace Datastore {
    /**
     * A type alias for kinds of transactions.
     */
    type TransactionType = 'transaction' | 'undo' | 'redo';
    /**
     * An options object for initializing a datastore.
     */
    interface IOptions {
        /**
         * The unique id of the datastore.
         */
        id: number;
        /**
         * The table schemas of the datastore.
         */
        schemas: ReadonlyArray<Schema>;
        /**
         * An optional handler for broadcasting transactions to peers.
         */
        adapter?: IServerAdapter;
        /**
         * An optional transaction id factory to override the default.
         */
        transactionIdFactory?: TransactionIdFactory;
        /**
         * Initialize the state to a previously serialized one.
         */
        restoreState?: string;
    }
    /**
     * The arguments object for the store `changed` signal.
     */
    interface IChangedArgs {
        /**
         * Whether the change was generated by transaction, undo, or redo.
         */
        readonly type: TransactionType;
        /**
         * The transaction id associated with the change.
         */
        readonly transactionId: string;
        /**
         * The id of the store responsible for the change.
         */
        readonly storeId: number;
        /**
         * A mapping of schema id to table change set.
         */
        readonly change: Change;
    }
    /**
     * A type alias for a store change.
     */
    type Change = {
        readonly [schemaId: string]: Table.Change<Schema>;
    };
    /**
     * A type alias for a store patch.
     */
    type Patch = {
        readonly [schemaId: string]: Table.Patch<Schema>;
    };
    /**
     * @internal
     */
    type MutableChange = {
        [schemaId: string]: Table.MutableChange<Schema>;
    };
    /**
     * @internal
     */
    type MutablePatch = {
        [schemaId: string]: Table.MutablePatch<Schema>;
    };
    /**
     * An object representing a datastore transaction.
     */
    type Transaction = {
        /**
         * The id of the transaction.
         */
        readonly id: string;
        /**
         * The id of the store responsible for the transaction.
         */
        readonly storeId: number;
        /**
         * The patch data of the transaction.
         */
        readonly patch: Patch;
        /**
         * The version of the source datastore.
         */
        readonly version: number;
    };
    /**
     * @internal
     */
    type Context = Readonly<Private.MutableContext>;
    /**
     * A factory function for generating a unique transaction id.
     */
    type TransactionIdFactory = (version: number, storeId: number) => string;
    /**
     * A helper function to wrap an update to the datastore in calls to
     * `beginTransaction` and `endTransaction`.
     *
     * @param datastore: the datastore to which to apply the update.
     *
     * @param update: A function that performs the update on the datastore.
     *   The function is called with a transaction id string, in case the
     *   user wishes to store the transaction ID for later use.
     *
     * @returns the transaction ID.
     *
     * #### Notes
     * If the datastore is already in a transaction, this does not attempt
     * to start a new one, and returns an empty string for the transaction
     * id. This allows for transactions to be composed a bit more easily.
     */
    function withTransaction(datastore: Datastore, update: (id: string) => void): string;
    /**
     * A base type for describing the location of data in a datastore,
     * to be consumed by some object. The only requirement is that it
     * has a datastore object. Objects extending from this will, in general,
     * have some combination of table, record, and field locations.
     */
    type DataLocation = {
        /**
         * The datastore in which the data is contained.
         */
        datastore: Datastore;
    };
    /**
     * An interface for referring to a specific table in a datastore.
     */
    type TableLocation<S extends Schema> = {
        /**
         * The schema in question. This schema must exist in the datastore,
         * or an error may result in its usage.
         */
        schema: S;
    };
    /**
     * An interface for referring to a specific record in a datastore.
     */
    type RecordLocation<S extends Schema> = TableLocation<S> & {
        /**
         * The record in question.
         */
        record: string;
    };
    /**
     * An interface for referring to a specific field in a datastore.
     *
     * #### Notes
     * The field must exist in the schema.
     */
    type FieldLocation<S extends Schema, F extends keyof S['fields']> = RecordLocation<S> & {
        /**
         * The field in question.
         */
        field: F;
    };
    /**
     * Get a given table by its location.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: The table location.
     *
     * @returns the table.
     */
    function getTable<S extends Schema>(datastore: Datastore, loc: TableLocation<S>): Table<S>;
    /**
     * Get a given record by its location.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: The record location.
     *
     * @returns the record, or undefined if it does not exist.
     */
    function getRecord<S extends Schema>(datastore: Datastore, loc: RecordLocation<S>): Record.Value<S> | undefined;
    /**
     * Get a given field by its location.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @returns the field in question.
     *
     * #### Notes
     * This will throw an error if the record does not exist in the given table.
     */
    function getField<S extends Schema, F extends keyof S['fields']>(datastore: Datastore, loc: FieldLocation<S, F>): S['fields'][F]['ValueType'];
    /**
     * Update a table.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: the table location.
     *
     * @param update: the update to the table.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateTable<S extends Schema>(datastore: Datastore, loc: TableLocation<S>, update: Table.Update<S>): void;
    /**
     * Update a record in a table.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: the record location.
     *
     * @param update: the update to the record.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateRecord<S extends Schema>(datastore: Datastore, loc: RecordLocation<S>, update: Record.Update<S>): void;
    /**
     * Update a field in a table.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @param update: the update to the field.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    function updateField<S extends Schema, F extends keyof S['fields']>(datastore: Datastore, loc: FieldLocation<S, F>, update: S['fields'][F]['UpdateType']): void;
    /**
     * Listen to changes in a table. Changes to other tables are ignored.
     *
     * @param datastore: the datastore in which the table resides.
     *
     * @param loc: the table location.
     *
     * @param slot: a callback function to invoke when the table changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenTable<S extends Schema>(datastore: Datastore, loc: TableLocation<S>, slot: (source: Datastore, args: Table.Change<S>) => void, thisArg?: any): IDisposable;
    /**
     * Listen to changes in a record in a table. Changes to other tables and
     * other records in the same table are ignored.
     *
     * @param datastore: the datastore in which the record resides.
     *
     * @param loc: the record location.
     *
     * @param slot: a callback function to invoke when the record changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenRecord<S extends Schema>(datastore: Datastore, loc: RecordLocation<S>, slot: (source: Datastore, args: Record.Change<S>) => void, thisArg?: any): IDisposable;
    /**
     * Listen to changes in a fields in a table. Changes to other tables, other
     * records in the same table, and other fields in the same record are ignored.
     *
     * @param datastore: the datastore in which the field resides.
     *
     * @param loc: the field location.
     *
     * @param slot: a callback function to invoke when the field changes.
     *
     * @returns an `IDisposable` that can be disposed to remove the listener.
     */
    function listenField<S extends Schema, F extends keyof S['fields']>(datastore: Datastore, loc: FieldLocation<S, F>, slot: (source: Datastore, args: S['fields'][F]['ChangeType']) => void, thisArg?: any): IDisposable;
}
declare namespace Private {
    /**
     * Validates all schemas, and throws an error if any are invalid.
     */
    function validateSchemas(schemas: ReadonlyArray<Schema>): void;
    /**
     * A three-way record comparison function.
     */
    function recordCmp<S extends Schema>(a: Table<S>, b: Table<S>): number;
    /**
     * A three-way record id comparison function.
     */
    function recordIdCmp<S extends Schema>(table: Table<S>, id: string): number;
    type MutableContext = {
        /**
         * Whether the datastore currently in a transaction.
         */
        inTransaction: boolean;
        /**
         * The id of the current transaction.
         */
        transactionId: string;
        /**
         * The current version of the datastore.
         */
        version: number;
        /**
         * The unique id of the datastore.
         */
        storeId: number;
        /**
         * The current change object of the transaction.
         */
        change: Datastore.MutableChange;
        /**
         * The current patch object of the transaction.
         */
        patch: Datastore.MutablePatch;
    };
    /**
     * Checks if a patch is empty.
     */
    function isPatchEmpty(patch: Datastore.Patch): boolean;
    /**
     * Checks if a change is empty.
     */
    function isChangeEmpty(change: Datastore.Change): boolean;
}
export {};
//# sourceMappingURL=datastore.d.ts.map