import { IIterable, IIterator, IterableOrArrayLike } from '@lumino/algorithm';
import { Datastore } from './datastore';
import { Record } from './record';
import { Schema } from './schema';
/**
 * A datastore object which holds a collection of records.
 */
export declare class Table<S extends Schema> implements IIterable<Record<S>> {
    /**
     * @internal
     *
     * Create a new datastore table.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     *
     * @returns A new datastore table.
     */
    static create<U extends Schema>(schema: U, context: Datastore.Context): Table<U>;
    /**
     * @internal
     *
     * Create a new datastore table with a previously exported state.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     *
     * @returns A new datastore table.
     */
    static recreate<U extends Schema>(schema: U, context: Datastore.Context, records: IterableOrArrayLike<Record<U>>): Table<U>;
    /**
     * @internal
     *
     * Apply a patch to a datastore table.
     *
     * @param table - The table of interest.
     *
     * @param data - The patch to apply to the table.
     *
     * @returns The user-facing change to the table.
     */
    static patch<U extends Schema>(table: Table<U>, data: Table.Patch<U>): Table.Change<U>;
    /**
     * @internal
     *
     * Unapply a patch to a datastore table, thereby undoing that patch.
     *
     * @param table - The table of interest.
     *
     * @param data - The patch to apply to the table.
     *
     * @returns The user-facing change to the table.
     */
    static unpatch<U extends Schema>(table: Table<U>, data: Table.Patch<U>): Table.Change<U>;
    /**
     * The schema for the table.
     *
     * #### Complexity
     * `O(1)`
     */
    readonly schema: S;
    /**
     * Whether the table is empty.
     *
     * #### Complexity
     * `O(1)`
     */
    get isEmpty(): boolean;
    /**
     * The size of the table.
     *
     * #### Complexity
     * `O(1)`
     */
    get size(): number;
    /**
     * Create an iterator over the records in the table.
     *
     * @returns A new iterator over the table records.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    iter(): IIterator<Record<S>>;
    /**
     * Test whether the table has a particular record.
     *
     * @param id - The id of the record of interest.
     *
     * @returns `true` if the table has the record, `false` otherwise.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    has(id: string): boolean;
    /**
     * Get the record for a particular id in the table.
     *
     * @param id - The id of the record of interest.
     *
     * @returns The record for the specified id, or `undefined` if no
     *   such record exists.
     *
     * #### Complexity
     * `O(log32 n)`
     */
    get(id: string): Record<S> | undefined;
    /**
     * Update one or more records in the table.
     *
     * @param data - The data for updating the records.
     *
     * #### Notes
     * If a specified record does not exist, it will be created.
     *
     * This method may only be called during a datastore transaction.
     */
    update(data: Table.Update<S>): void;
    /**
     * Construct a new datastore table.
     *
     * @param schema - The schema for the table.
     *
     * @param context - The datastore context.
     */
    private constructor();
    private _context;
    private _records;
}
/**
 * The namespace for the `Table` class statics.
 */
export declare namespace Table {
    /**
     * A type alias for the table update type.
     */
    type Update<S extends Schema> = {
        readonly [recordId: string]: Record.Update<S>;
    };
    /**
     * A type alias for the table change type.
     */
    type Change<S extends Schema> = {
        readonly [recordId: string]: Record.Change<S>;
    };
    /**
     * A type alias for the table patch type.
     */
    type Patch<S extends Schema> = {
        readonly [recordId: string]: Record.Patch<S>;
    };
    /**
     * @internal
     *
     * A type alias for the table mutable change type.
     */
    type MutableChange<S extends Schema> = {
        [recordId: string]: Record.MutableChange<S>;
    };
    /**
     * @internal
     *
     * A type alias for the table mutable patch type.
     */
    type MutablePatch<S extends Schema> = {
        [recordId: string]: Record.MutablePatch<S>;
    };
}
//# sourceMappingURL=table.d.ts.map