import { Field } from './field';
/**
 * A field which represents collaborative text.
 */
export declare class TextField extends Field<TextField.Value, TextField.Update, TextField.Metadata, TextField.Change, TextField.Patch> {
    /**
     * Construct a new text field.
     *
     * @param options - The options for initializing the field.
     */
    constructor(options?: TextField.IOptions);
    /**
     * The discriminated type of the field.
     */
    get type(): 'text';
    /**
     * Create the initial value for the field.
     *
     * @returns The initial value for the field.
     */
    createValue(): TextField.Value;
    /**
     * Create the metadata for the field.
     *
     * @returns The metadata for the field.
     */
    createMetadata(): TextField.Metadata;
    /**
     * Apply a user update to the field.
     *
     * @param args - The arguments for the update.
     *
     * @returns The result of applying the update.
     */
    applyUpdate(args: Field.UpdateArgs<TextField.Value, TextField.Update, TextField.Metadata>): Field.UpdateResult<TextField.Value, TextField.Change, TextField.Patch>;
    /**
     * Apply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of applying the patch.
     */
    applyPatch(args: Field.PatchArgs<TextField.Value, TextField.Patch, TextField.Metadata>): Field.PatchResult<TextField.Value, TextField.Change>;
    /**
     * Unapply a system patch to the field.
     *
     * @param args - The arguments for the patch.
     *
     * @returns The result of unapplying the patch.
     */
    unapplyPatch(args: Field.PatchArgs<TextField.Value, TextField.Patch, TextField.Metadata>): Field.PatchResult<TextField.Value, TextField.Change>;
    /**
     * Merge two change objects into a single change object.
     *
     * @param first - The first change object of interest.
     *
     * @param second - The second change object of interest.
     *
     * @returns A new change object which represents both changes.
     */
    mergeChange(first: TextField.Change, second: TextField.Change): TextField.Change;
    /**
     * Merge two patch objects into a single patch object.
     *
     * @param first - The first patch object of interest.
     *
     * @param second - The second patch object of interest.
     *
     * @returns A new patch object which represents both patches.
     */
    mergePatch(first: TextField.Patch, second: TextField.Patch): TextField.Patch;
}
/**
 * The namespace for the `TextField` class statics.
 */
export declare namespace TextField {
    /**
     * An options object for initializing a text field.
     */
    interface IOptions extends Field.IOptions {
    }
    /**
     * A type alias for the value type of a text field.
     */
    type Value = string;
    /**
     * A type alias for a text field splice.
     */
    type Splice = {
        /**
         * The index of the splice.
         */
        readonly index: number;
        /**
         * The number of characters to remove.
         */
        readonly remove: number;
        /**
         * The text to insert.
         */
        readonly text: string;
    };
    /**
     * A type alias for the text field update type.
     */
    type Update = Splice | ReadonlyArray<Splice>;
    /**
     * A type alias for the text field metadata type.
     */
    type Metadata = {
        /**
         * An array of ids corresponding to the text characters.
         */
        readonly ids: Array<string>;
        /**
         * The cemetery for concurrently deleted characters.
         */
        readonly cemetery: {
            [id: string]: number;
        };
    };
    /**
     * A type alias for a text field change part.
     */
    type ChangePart = {
        /**
         * The index of the modification.
         */
        readonly index: number;
        /**
         * The text that was removed.
         */
        readonly removed: string;
        /**
         * The text that was inserted.
         */
        readonly inserted: string;
    };
    /**
     * A type alias for the text field change type.
     */
    type Change = ReadonlyArray<ChangePart>;
    /**
     * A type alias for the text field patch part.
     */
    type PatchPart = {
        /**
         * The ids that were removed.
         */
        readonly removedIds: ReadonlyArray<string>;
        /**
         * The text that was removed.
         */
        readonly removedText: string;
        /**
         * The ids that were inserted.
         */
        readonly insertedIds: ReadonlyArray<string>;
        /**
         * The text that was inserted.
         */
        readonly insertedText: string;
    };
    /**
     * A type alias for the text field patch type.
     */
    type Patch = ReadonlyArray<PatchPart>;
}
//# sourceMappingURL=textfield.d.ts.map